# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

from autofactory.django.factory import DjangoModelAutoFactory
from autofactory.shortcuts import autofactory

__author__ = "Nick Gashkov"
__email__ = "nick@gashkov.com"
__version__ = "0.1.0"

__all__ = (
    "DjangoModelAutoFactory",
    "autofactory",
)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

from factory import DjangoModelFactory

from autofactory.django.options import DjangoAutoOptions


class DjangoModelAutoFactory(DjangoModelFactory):
    _options_class = DjangoAutoOptions



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

from factory.base import FactoryMetaClass


def build_django_autofactory(model_cls, **kwargs):
    from autofactory.django.factory import DjangoModelAutoFactory

    class Meta:
        model = model_cls
        fields = "__all__"

    factory_cls_name = "Generated" + model_cls.__name__ + "Factory"
    factory_cls_name = str(factory_cls_name)

    factory_attrs = kwargs.copy()
    factory_attrs["Meta"] = Meta

    return FactoryMetaClass(
        factory_cls_name,
        (DjangoModelAutoFactory,),
        factory_attrs,
    )



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

from factory.base import OptionDefault
from factory.django import DjangoOptions

from autofactory.django import compat
from autofactory.django.introspector import DjangoIntrospector


class DjangoAutoOptions(DjangoOptions):
    _introspector_class = DjangoIntrospector

    @property
    def declarations(self):
        declarations = super(DjangoAutoOptions, self).declarations
        declarations.update(self.get_autodeclarations(declarations))

        return declarations

    def get_introspector(self):
        return self._introspector_class(self.model)

    def get_autodeclarations(self, declarations):
        introspecter = self.get_introspector()

        autofields = self.get_fields_to_autodeclare()
        autofields = [f for f in autofields if f.name not in declarations]

        return introspecter.build_all(autofields)

    def get_fields_to_autodeclare(self):
        all_fields = self._get_all_fields()

        if self.fields == "__all__":
            return self._get_not_blank_fields(all_fields)

        return filter(lambda x: x.name in self.fields, all_fields)

    def _build_default_options(self):
        return super(DjangoAutoOptions, self)._build_default_options() + [
            OptionDefault("fields", tuple(), inherit=True),
        ]

    def _get_all_fields(self):
        return compat.get_all_fields(self.model)

    def _get_not_blank_fields(self, fields):
        return compat.get_not_blank_fields(fields)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

from autofactory.django import builders, compat


class DjangoIntrospector(object):
    def __init__(self, model):
        self.model = model

    def build_all(self, fields):
        declarations = dict()

        for field in filter(self._is_concrete_field, fields):
            declarations[field.name] = self.build(field)

        return declarations

    def build(self, field):
        builder = self.get_builder(field)
        built = builder and builder(field)

        return built

    def get_builder(self, field):
        builder = self._get_concrete_builder(field)

        if builder is not None:
            return builder

        return self._get_generic_builder(field)

    def _get_concrete_builder(self, field):
        if getattr(field, "choices", tuple()):
            return builders.from_choices

        if field.has_default():
            return builders.from_default

        return None

    def _get_generic_builder(self, field):
        field_cls = type(field)

        builder_name = "build_" + field_cls.__name__.lower()
        builder = getattr(builders, builder_name, None)

        if builder is None:
            raise TypeError(
                "'{field_cls}' is not supported.".format(
                    field_cls=field_cls.__name__,
                )
            )

        return builder

    def _is_concrete_field(self, field):
        return field.__class__ not in compat.get_generic_fields()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

import factory


def build_charfield(field_cls):
    return factory.Faker("text", max_nb_chars=field_cls.max_length)


def build_emailfield(field_cls):
    return factory.Faker("email")


def build_genericipaddressfield(field_cls):
    return factory.Faker("ipv4")


def build_slugfield(field_cls):
    return factory.Faker("slug")


def build_textfield(field_cls):
    return factory.Faker("text")


def build_urlfield(field_cls):
    return factory.Faker("url")


def build_uuidfield(field_cls):
    return factory.Faker("uuid4")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

import factory

# `factory.Faker("pyint")` is a random integer ranges from 0 to 9999, so it
# will be good enough for every integer-related field.


def build_bigintegerfield(field_cls):
    return factory.Faker("pyint")


def build_decimalfield(field_cls):
    characteristic = field_cls.max_digits - field_cls.decimal_places
    mantissa = field_cls.decimal_places

    return factory.Faker(
        "pydecimal",
        left_digits=characteristic,
        right_digits=mantissa,
    )


def build_floatfield(field_cls):
    return factory.Faker("pyfloat")


def build_integerfield(field_cls):
    return factory.Faker("pyint")


def build_positiveintegerfield(field_cls):
    return factory.Faker("pyint")


def build_positivesmallintegerfield(field_cls):
    return factory.Faker("pyint")


def build_smallintegerfield(field_cls):
    return factory.Faker("pyint")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

from autofactory.django.builders.booleans import (
    build_booleanfield,
    build_nullbooleanfield,
)
from autofactory.django.builders.concrete import from_choices, from_default
from autofactory.django.builders.datetimes import (
    build_datefield,
    build_datetimefield,
    build_durationfield,
    build_timefield,
)
from autofactory.django.builders.misc import (
    build_binaryfield,
    build_filefield,
    build_filepathfield,
    build_imagefield,
)
from autofactory.django.builders.numbers import (
    build_bigintegerfield,
    build_decimalfield,
    build_floatfield,
    build_integerfield,
    build_positiveintegerfield,
    build_positivesmallintegerfield,
    build_smallintegerfield,
)
from autofactory.django.builders.relationships import (
    build_foreignkey,
    build_manytomanyfield,
    build_onetoonefield,
)
from autofactory.django.builders.strings import (
    build_charfield,
    build_emailfield,
    build_genericipaddressfield,
    build_slugfield,
    build_textfield,
    build_urlfield,
    build_uuidfield,
)

__all__ = (
    "build_bigintegerfield",
    "build_binaryfield",
    "build_booleanfield",
    "build_charfield",
    "build_datefield",
    "build_datetimefield",
    "build_decimalfield",
    "build_durationfield",
    "build_emailfield",
    "build_filefield",
    "build_filepathfield",
    "build_floatfield",
    "build_imagefield",
    "build_integerfield",
    "build_genericipaddressfield",
    "build_nullbooleanfield",
    "build_positiveintegerfield",
    "build_positivesmallintegerfield",
    "build_slugfield",
    "build_smallintegerfield",
    "build_textfield",
    "build_timefield",
    "build_urlfield",
    "build_uuidfield",
    "build_foreignkey",
    "build_manytomanyfield",
    "build_onetoonefield",
    "from_choices",
    "from_default",
)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

import django.db.models

django_version = django.VERSION[:2]


def get_all_fields(model):
    if django_version >= (1, 8):
        return model._meta._get_fields(reverse=False)

    return [
        field for field, __ in model._meta.get_fields_with_model()
    ] + [
        field for field, __ in model._meta.get_m2m_with_model()
    ]


def get_generic_fields():
    generic_fields = [
        django.db.models.AutoField,
        django.db.models.ManyToOneRel,
        django.db.models.ManyToManyRel,
    ]

    if django_version >= (2, 0):
        generic_fields.append(django.db.models.BigAutoField)

    return generic_fields


def get_related_model(field_cls):
    if django_version >= (2, 0):
        return field_cls.remote_field.model

    return field_cls.rel.to


def get_not_blank_fields(fields):
    boolean_fields = {
        django.db.models.BooleanField,
        django.db.models.NullBooleanField,
    }

    return [f for f in fields if f.__class__ in boolean_fields or not f.blank]



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

from autofactory.django.shortcuts import build_django_autofactory


def autofactory(instance):
    return build_django_autofactory(instance)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

import factory


def build_binaryfield(field_cls):
    return factory.Faker("binary", length=(1 * 1024))


def build_filefield(field_cls):
    return factory.django.FileField()


def build_filepathfield(field_cls):
    return factory.Faker("file_path")


def build_imagefield(field_cls):
    return factory.django.ImageField()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

import factory


def build_datefield(field_cls):
    return factory.Faker("date_this_decade")


def build_datetimefield(field_cls):
    return factory.Faker("date_time")


def build_durationfield(field_cls):
    return factory.Faker("time_delta")


def build_timefield(field_cls):
    return factory.Faker("time_object")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

import factory

from autofactory.django.compat import get_related_model
from autofactory.django.shortcuts import build_django_autofactory


def build_foreignkey(field_cls):
    model = get_related_model(field_cls)
    model_factory = build_django_autofactory(model)

    return factory.SubFactory(model_factory)


def build_manytomanyfield(field_cls):
    field_descriptor = getattr(field_cls.model, field_cls.name)
    field_through = field_descriptor.through

    if not field_through._meta.auto_created:
        return build_manytomanyfield_with_a_through(field_cls)

    model_factory = build_django_autofactory(get_related_model(field_cls))

    def builder(obj, create, _extracted, **_kwargs):
        if not create:
            return

        manager = getattr(obj, field_cls.name)
        manager.add(*model_factory.create_batch(2))

    return factory.PostGeneration(builder)


def build_manytomanyfield_with_a_through(field_cls):
    field_descriptor = getattr(field_cls.model, field_cls.name)
    field_through = field_descriptor.through

    field_through_factory = build_django_autofactory(field_through, **{field_cls.m2m_field_name(): None})

    return factory.RelatedFactory(field_through_factory, field_cls.m2m_field_name())


def build_onetoonefield(field_cls):
    model = get_related_model(field_cls)
    model_factory = build_django_autofactory(model)

    return factory.SubFactory(model_factory)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

import os
import re

from setuptools import setup, find_packages


def get_version():
    py = open(os.path.join("autofactory", "__init__.py")).read()
    py_version = re.search("__version__ = ['\"]([^'\"]+)['\"]", py).group(1)

    return py_version


with open("README.md") as readme_file:
    readme = readme_file.read()

install_requires = [
    "factory_boy>=2.11.0"
]

setup(
    author="Nick Gashkov",
    author_email="nick@gashkov.com",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Testing",
    ],
    description="AutoFactoryBoy generates factories for you.",
    install_requires=install_requires,
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="autofactory",
    name="autofactory",
    packages=find_packages(exclude=["tests", "tests.*"]),
    url="https://github.com/nickgashkov/autofactoryboy",
    version=get_version(),
    zip_safe=False,
)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018-2019 Nick Gashkov
#
# Distributed under MIT License. See LICENSE file for details.
from __future__ import unicode_literals

from factory import random


def from_choices(field_cls):
    random_choice_tuple = random.randgen.choice(field_cls.choices)
    random_choice = random_choice_tuple[0]

    return random_choice


def from_default(field_cls):
    return field_cls.default
